@tool
extends Node3D

const STEEPNESS = 0.02

@export var rebake: bool :
	set(_value): _rebake()

func _ready() -> void:
	_rebake()

func _rebake() -> void:
	var nav_region := get_node_or_null(^"NavigationRegion3D")
	if !nav_region:
		nav_region = NavigationRegion3D.new()
		nav_region.name = &"NavigationRegion3D"
		add_child(nav_region, true)
		nav_region.owner = self
	
	var mesh_instance := get_node_or_null(^"MeshInstance3D")
	if !mesh_instance:
		mesh_instance = MeshInstance3D.new()
		mesh_instance.name = &"MeshInstance3D"
		add_child(mesh_instance, true)
		mesh_instance.owner = self
	
	var nav_mesh := NavigationMesh.new()
	
	var map := get_world_3d().navigation_map
	assert(NavigationServer3D.map_get_cell_size(map) == nav_mesh.cell_size)
	assert(NavigationServer3D.map_get_cell_height(map) == nav_mesh.cell_height)
	
	# Create a parabolic U-shape mesh
	var st := SurfaceTool.new()
	st.begin(Mesh.PRIMITIVE_TRIANGLES)
	st.set_uv(Vector2.ZERO)
	for x in range(-32, 32):
		for z in range(-32, 32):
			st.add_vertex(_vertex(x+0, z+0))
			st.add_vertex(_vertex(x+1, z+0))
			st.add_vertex(_vertex(x+0, z+1))
			st.add_vertex(_vertex(x+1, z+0))
			st.add_vertex(_vertex(x+1, z+1))
			st.add_vertex(_vertex(x+0, z+1))
	st.index()
	st.generate_normals()
	st.generate_tangents()
	mesh_instance.mesh = st.commit()
	
	# Bake a navmesh for the U-shaped mesh
	var source_geometry_data := NavigationMeshSourceGeometryData3D.new()
	source_geometry_data.add_mesh(mesh_instance.mesh, Transform3D.IDENTITY)
	NavigationMeshGenerator.bake_from_source_geometry_data(nav_mesh, source_geometry_data)
	
	nav_region.navigation_mesh = nav_mesh

func _vertex(x: float, z: float) -> Vector3:
	return Vector3(x, _get_height(x, z), z)

func _get_height(x: float, z: float) -> float:
	return (x-z) * (x-z) * STEEPNESS
